import serial
import sys
import re
import datetime
import getopt
import time
from enum import Enum, auto

verbose = False

class ATRTRN(Enum):
# Common
    PASS = auto()
    FAIL = auto()
    PORT_TIMEOUT = auto()

# at+csq?
    READ_SIGNAL_FAIL = auto()
    NO_SIGNAL  = auto()
    BAD_SIGNAL = auto()

# at+cpin?
    READ_SIM_FIAL = auto()
    NO_SIM = auto()

# AT+CEREG?
    READ_REG_FIAL = auto()
    FINDING_NET = auto()
    REJECT_NET = auto()
    UNKOWN_NET = auto()

# AT+CFUN?
    RF_SIM_NOT_WORK = auto()
    RF_NOT_WORK = auto()

# AT+COPS?
    MANUAL_OPERATOR = auto()
    UNKOWN_OPERATOR_ISSUE = auto()

# Common
    UNKOWN_ISSUE = auto()
    MISSMATCH = auto()

def print_error(message):
    t_time = datetime.datetime.now().strftime("%Y/%m/%d-%H:%M:%S")
    print(f"[{t_time}]\t | [ERROR] | {message}")

def print_info(message):
    t_time = datetime.datetime.now().strftime("%Y/%m/%d-%H:%M:%S")
    print(f"[{t_time}]\t | [info] | {message}")

def verbose_error(message):
    if verbose:
        print_error(message)

def verbose_info(message):
    if verbose:
        print_info(message)

def send_at_command(ser, command):
    ser.write((command + '\r').encode())
    time.sleep(0.2)
    response = ser.read(ser.in_waiting or 1).decode()
    verbose_info(f"{command} --> {response.strip()}")
    return response.strip()

def is_pass(rtrn):
    if ATRTRN.PASS == rtrn:
        verbose_info(rtrn)
        return 0
    else:
        verbose_error(rtrn)
        return 1

def check_signal(ser, cmd):
    msg = send_at_command(ser, cmd)
    if msg != None:
        matches = re.findall(r'\d+', msg)
        if matches:
            signal = int(matches[0])
            if signal == 99:
                verbose_error("No signal")
                return ATRTRN.NO_SIGNAL
            elif signal < 12 :
                verbose_error("Bad signal")
                return ATRTRN.BAD_SIGNAL
            elif signal >=12 and signal < 20:
                verbose_info("Weak signal")
                return ATRTRN.PASS
            else:
                return ATRTRN.PASS
        else:
            return ATRTRN.MISSMATCH
    else:
        return ATRTRN.PORT_TIMEOUT

def check_sim_card(ser, cmd):
    msg = send_at_command(ser, cmd)
    if msg != None:
        if "READY" in msg:
            verbose_info("SIM Card is working well")
            return ATRTRN.PASS
        else:
            verbose_error("Not Found the SIM Card")
            return ATRTRN.NO_SIM
    else:
        return ATRTRN.PORT_TIMEOUT

def check_network_registration_status(ser, cmd):
    msg = send_at_command(ser, cmd)
    if msg != None:
        matches = re.findall(r'\d+', msg)
        if matches:
            n = int(matches[0])
            stat = int(matches[1])
            if n == 0:
                if stat == 0:
                    verbose_error("Not registered. MT is not currently searching an operator to register to")
                    return ATRTRN.UNKOWN_ISSUE
                elif stat == 1:
                    verbose_info("Registered, home network")
                    return ATRTRN.PASS
                elif stat == 2:
                    verbose_error("Not registered, but MT is currently trying to attach or searching an operator to register to")
                    return ATRTRN.FINDING_NET
                elif stat == 3:
                    verbose_error("Registration denied")
                    return ATRTRN.REJECT_NET
                elif stat == 4:
                    verbose_error("Unknown")
                    return ATRTRN.UNKOWN_NET
                elif stat == 5:
                    verbose_info("Registered, roaming")
                    return ATRTRN.PASS
                else:
                    return ATRTRN.UNKOWN_ISSUE
            else:
                return ATRTRN.UNKOWN_ISSUE
        else:
            return ATRTRN.MISSMATCH
    else:
        return ATRTRN.PORT_TIMEOUT

def check_functionality(ser, cmd):
    msg = send_at_command(ser, cmd)
    if msg != None:
        matches = re.findall(r'\d+', msg)
        if matches:
            func = int(matches[0])
            if func == 0:
                verbose_error("RF and (U)SIM card not working")
                return ATRTRN.RF_SIM_NOT_WORK
            elif func == 1:
                verbose_info("Full functionality")
                return ATRTRN.PASS
            elif func == 4:
                verbose_error("Disable the ME from both transmitting and receiving RF signals")
                return ATRTRN.RF_NOT_WORK
            else:
                return ATRTRN.UNKOWN_ISSUE
        else:
            return ATRTRN.MISSMATCH
    else:
        return ATRTRN.PORT_TIMEOUT

def check_operator_selection(ser, cmd):
    msg = send_at_command(ser, cmd)
    if msg != None:
        matches = re.findall(r'\d+', msg)
        if matches:
            func = int(matches[0])
            if func == 0:
                verbose_info("Automatic mode")
                return ATRTRN.PASS
            elif func == 1:
                verbose_info("Manual operator selection")
                return ATRTRN.MANUAL_OPERATOR
            else:
                return ATRTRN.UNKOWN_OPERATOR_ISSUE
        else:
            return ATRTRN.MISSMATCH
    else:
        return ATRTRN.PORT_TIMEOUT


if __name__ == "__main__":
    rtrn=254
    argv = sys.argv[1:]
    opts, args = getopt.getopt(argv, "vd:c:")
    for opt, arg in opts:
        if opt in ['-v']:
            verbose = True
        elif opt in ['-d']:
            port=arg
        elif opt in ['-c']:
            at_cmd=arg.upper()
        else:
            sys.exit(100)
  
    ser = serial.Serial(port, 115200, timeout=0.4)
    ser.reset_input_buffer()
    ser.reset_output_buffer()
    
    try:
        if at_cmd == "AT+CSQ":
            rtrn = is_pass(check_signal(ser, at_cmd))
        elif at_cmd == "AT+CPIN?":
            rtrn = is_pass(check_sim_card(ser, at_cmd))
        elif at_cmd == "AT+CEREG?":
            rtrn = is_pass(check_network_registration_status(ser, at_cmd))
        elif at_cmd == "AT+CFUN?":
            rtrn = is_pass(check_functionality(ser, at_cmd))
        elif at_cmd == "AT+COPS?":
            rtrn = is_pass(check_operator_selection(ser, at_cmd))        
        else:
            response_msg = send_at_command(ser, at_cmd)
            if verbose == False:
                print_info(at_cmd)
                print_info(response_msg)
            rtrn = 0
    finally:
        ser.close()            
        sys.exit(rtrn)
